#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>

#include <Arduino.h>
#include <IRremoteESP8266.h>
#include <IRac.h>
#include <IRutils.h>

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>



#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


const uint16_t kIrLed = 12;  // The ESP GPIO pin to use that controls the IR LED.
IRac ac(kIrLed);  // Create a A/C object using GPIO to sending messages with.

String  DeviceName = "Control Room";
String  LocalIP;


//Variables
int i = 0;
int statusCode;
const char* ssid = "text";
const char* passphrase = "text";

const char* devicename = "text";

String st;
String content;


//Function Declaration
bool testWifi(void);
void launchWeb(void);
void setupAP(void);

//--------Establishing Local server at port 80 whenever required
ESP8266WebServer server0(80);


//Specifying the boolean variables indicating the status of LED1 to LED4
bool AC_status = false, Mode_status = false, Turbo_status = false,  Temp_status = false,Test_status = false;
int Fan_status = 0;
int SwingUD_status = 0;

int ProList = 0;
int protocolfromE=0;
int hotcoldE=0;
int speedE=0;

void setup()
{
  Serial.begin(115200); //Initializing if(DEBUG)Serial Monitor

  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;); // Don't proceed, loop forever
  }
  
  display.clearDisplay();
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(22, 40);            // Start at top-left corner
  display.println("connecting");
  display.display();


  // Set up what we want to send.
  // See state_t, opmode_t, fanspeed_t, swingv_t, & swingh_t in IRsend.h for
  // all the various options.
  ac.next.protocol = decode_type_t::GREE;  // Set a protocol to use.
  ac.next.model = 1;  // Some A/Cs have different models. Try just the first.
  ac.next.mode = stdAc::opmode_t::kCool;  // Run in cool mode initially.
  ac.next.celsius = true;  // Use Celsius for temp units. False = Fahrenheit
  ac.next.degrees = 30;  // 25 degrees.
  ac.next.fanspeed = stdAc::fanspeed_t::kLow;  // Start the fan at medium.
  ac.next.swingv = stdAc::swingv_t::kOff;  // Don't swing the fan up or down.
  ac.next.swingh = stdAc::swingh_t::kOff;  // Don't swing the fan left or right.
  ac.next.light = true;  // Turn off any LED/Lights/Display that we can.
  ac.next.beep = true;  // Turn off any beep from the A/C if we can.
  ac.next.econo = false;  // Turn off any economy modes if we can.
  ac.next.filter = false;  // Turn off any Ion/Mold/Health filters if we can.
  ac.next.turbo = false;  // Don't use any turbo/powerful/etc modes.
  ac.next.quiet = false;  // Don't use any quiet/silent/etc modes.
  ac.next.sleep = -1;  // Don't set any sleep time or modes.
  ac.next.clean = false;  // Turn off any Cleaning options if we can.
  ac.next.clock = -1;  // Don't set any current time if we can avoid it.
  ac.next.power = false;  // Initially start with the unit off.


  Serial.println();
  Serial.println("Disconnecting previously connected WiFi");
  WiFi.disconnect();
  EEPROM.begin(512); //Initializing EEPROM
  delay(10);
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.println();
  Serial.println();
  Serial.println("Startup");

  // Read eeprom for ssid and password
  Serial.println("Reading EEPROM ssid");

  String esid;
  for (int i = 0; i < 32; ++i)
  {
    esid += char(EEPROM.read(i));
  }

  Serial.println();
  Serial.print("SSID: ");
  Serial.println(esid);
  Serial.println("Reading EEPROM pass");

  String epass = "";
  for (int i = 32; i < 96; ++i)
  {
    epass += char(EEPROM.read(i));
  }
  Serial.print("PASS: ");
  Serial.println(epass);

  String enameset;
  for (int i = 96; i < 160; ++i)
  {
    enameset += char(EEPROM.read(i));
  }
  Serial.print("NAME: ");
  Serial.println(enameset);

  DeviceName = enameset;
  
  ProList = EEPROM.read(160);

  hotcoldE = EEPROM.read(161);
  if(hotcoldE ==1){
    Mode_status = false;
  }
  if(hotcoldE ==0){
    Mode_status = true;
  }

  Fan_status = EEPROM.read(162);
  
  Turbo_status = EEPROM.read(163);
   if(Turbo_status == 0){
    Turbo_status == false;
  } 
     if(Turbo_status == 1){
    Turbo_status == true;
  } 
    SwingUD_status = EEPROM.read(164);
    ac.next.degrees = EEPROM.read(165);
  
  WiFi.begin(esid.c_str(), epass.c_str());
  if (testWifi())
  {
    Serial.println("Succesfully Connected!!!");

    Serial.println(WiFi.localIP());
    LocalIP = String() + WiFi.localIP()[0] + "." + WiFi.localIP()[1] + "." + WiFi.localIP()[2] + "." + WiFi.localIP()[3];
    
    
  display.clearDisplay();


  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(22, 20);            // Start at top-left corner
  display.println(DeviceName);
  
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(22, 40);            // Start at top-left corner
  display.println(LocalIP);
  display.display();
    
    
    server0.on("/", handle_OnConnect);
    server0.on("/ACon", handle_ACon);
    server0.on("/ACoff", handle_ACoff);

    server0.on("/heat", handle_heat);
    server0.on("/cool", handle_cool) ;

    server0.on("/low", handle_low);
    server0.on("/medium", handle_medium);
    server0.on("/high", handle_high);

    server0.on("/normal", handle_normal);
    server0.on("/turbo", handle_turbo);

    server0.on("/swingOff", handle_swingOff);
    server0.on("/swingLow", handle_swingLow);
    //  server.on("/swingMed",handle_swingMed);
    server0.on("/swingH", handle_swingH);

    server0.on("/up", handle_up);
    server0.on("/down", handle_down);

    server0.on("/set", setProto);
 
//    server0.on("/test", TestACIR);
//    server0.on("/stoptest", StopTestACIR);
//    
    server0.onNotFound(handle_NotFound);


    server0.begin();
    //Starting the Server
    Serial.println("HTTP Server Started");
    WiFi.softAPdisconnect(true);

    return;



  }
  else
  {
    Serial.println("Turning the HotSpot On");
    launchWeb();
    setupAP();// Setup accesspoint or HotSpot
  }

  Serial.println();
  Serial.println("Waiting.");

  while ((WiFi.status() != WL_CONNECTED))
  {
    Serial.print(".");
    delay(100);
    server0.handleClient();
  }


  //Specifying the functions which will be executed upon corresponding GET request from the client

}
void loop() {
  if (WiFi.status() == WL_CONNECTED) //Check if ESP8266 is still connected to the internet
  {
    //Assign the server to handle the clients
    server0.handleClient();
  }
  else
  {
    Serial.println("WiFi Disconnected!!!");
    Serial.print("Trying to establish the connection...");

    //Keep checking the WiFi status until it is connected to the wifi network
    while (WiFi.status() != WL_CONNECTED)
    {
      delay(1000);
      Serial.print(".");
    }
    Serial.println("");
    Serial.print("WiFi Connected with IP Address: ");
    Serial.println(WiFi.localIP());

  }
}


//Functions used for saving WiFi credentials and to connect to it which you do not need to change
bool testWifi(void)
{
  int c = 0;
  Serial.println("Waiting for WiFi to connect");
  while ( c < 240 ) {
    if (WiFi.status() == WL_CONNECTED)
    {
      return true;
    }
    delay(500);
    Serial.print("*");
    c++;
  }
  Serial.println("");
  Serial.println("Connection timed out, opening AP or Hotspot");
  return false;
}

void launchWeb()
{
  Serial.println("");
  if (WiFi.status() == WL_CONNECTED)
    Serial.println("WiFi connected");
  Serial.print("Local IP: ");
  Serial.println(WiFi.localIP());
  Serial.print("SoftAP IP: ");
  Serial.println(WiFi.softAPIP());

  IPAddress ip = WiFi.softAPIP();
  String ipStr = String(ip[0]) + '.' + String(ip[1]) + '.' + String(ip[2]) + '.' + String(ip[3]);

  display.clearDisplay();
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(22, 20);            // Start at top-left corner
  display.println("WiFI Setup");
  
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE);        // Draw white text
  display.setCursor(22, 40);            // Start at top-left corner
  display.println(ipStr);
  display.display();

  createWebServer();
  // Start the server
  server0.begin();
  Serial.println("Server started");
}

void setupAP(void)
{
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  int n = WiFi.scanNetworks();
  Serial.println("scan completed");
  if (n == 0)
    Serial.println("No WiFi Networks found");
  else
  {
    Serial.print(n);
    Serial.println(" Networks found");
    for (int i = 0; i < n; ++i)
    {
      // Print SSID and RSSI for each network found
      Serial.print(i + 1);
      Serial.print(": ");
      Serial.print(WiFi.SSID(i));
      Serial.print(" (");
      Serial.print(WiFi.RSSI(i));
      Serial.print(")");
      Serial.println((WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " " : "*");
      delay(10);
    }
  }
  Serial.println("");
  st = "<ol>";
  for (int i = 0; i < n; ++i)
  {
    // Print SSID and RSSI for each network found
    st += "<li>";
    st += WiFi.SSID(i);
    st += " (";
    st += WiFi.RSSI(i);

    st += ")";
    st += (WiFi.encryptionType(i) == ENC_TYPE_NONE) ? " " : "*";
    st += "</li>";
  }
  st += "</ol>";
  delay(100);
  WiFi.softAP("AC IR REMOTE AP", "");
  Serial.println("Initializing_Wifi_accesspoint");
  launchWeb();
  Serial.println("over");
}

void createWebServer()
{
  {
    server0.on("/", []() {

      IPAddress ip = WiFi.softAPIP();
      String ipStr = String(ip[0]) + '.' + String(ip[1]) + '.' + String(ip[2]) + '.' + String(ip[3]);

      //        display.setTextSize(1);             // Normal 1:1 pixel scale
      //  display.setTextColor(WHITE);        // Draw white text
      //  display.setCursor(22, 40);            // Start at top-left corner
      //  display.println(ipStr);
      //display.display();

      content = "<!DOCTYPE html> <html>\n";
      content += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
      content += "<title>AC Remote Server</title>\n";
      content += "<style>html{font-family:Helvetica; display:inline-block; margin:0px auto; text-align:center;}\n";
      content += "body{margin-top: 15px;} h1{color: #444444; margin: 10px auto 25px;} h2{color: #444444; margin-bottom: 50px; font-size:15px; margin-top: 5px;}  h3{color:#444444; margin-bottom: 0px;} \n";


      content += ".button{display:block; width:80px; background-color:#f48100; border:none; color:white; padding: 3px 15px; text-decoration:none; font-size:17px; margin: 4px auto 20px; cursor:pointer; border-radius:4px;}\n";

      content += ".button-submit{background-color:#008000;}\n";
      content += ".button-submit:active{background-color:#008000;}\n";

      content += "</style>\n";
      content += "</head>\n";
      content += "<body>\n";
      content += "<p><h1>AC Remote Wifi Connection</h1></a>\n";

      content += "<form action=\"/scan\" method=\"POST\"><input type=\"submit\" value=\"scan\"></form>";
      content += "<p></a>\n";

      content += ipStr;
      content += "<p>";
      content += st;
      
      content += "</p><form method='get' action='setting'></a>\n";    
     content += " <label>DEVICE NAME: </label> </p> <input name='devicename' length=32> </p> <label>SSID: </label> </p> <input name='ssid' length=32> </p> <label>  PASS: </label> </p>  <input name='pass' length=64> </p> <input type ='submit'> </form>";



   
      content += "</html>";
      server0.send(200, "text/html", content);
    });
    server0.on("/scan", []() {
      setupAP();
      IPAddress ip = WiFi.softAPIP();
      String ipStr = String(ip[0]) + '.' + String(ip[1]) + '.' + String(ip[2]) + '.' + String(ip[3]);

      content = "<!DOCTYPE HTML>\r\n<html>go back and refresh";
      server0.send(200, "text/html", content);
    });

    server0.on("/setting", []() {

      String qsid = server0.arg("ssid");
      String qpass = server0.arg("pass");
      String qnameset = server0.arg("devicename");

      if (qsid.length() > 0 && qpass.length() > 0 && qnameset.length() > 0) {
        Serial.println("clearing eeprom");

        for (int i = 0; i < 160; ++i) {
          EEPROM.write(i, 0);
        }

        Serial.println(qsid);
        Serial.println("");
        Serial.println(qpass);
        Serial.println("");

        Serial.println("writing eeprom ssid:");
        for (int i = 0; i < qsid.length(); ++i)
        {
          EEPROM.write(i, qsid[i]);
          Serial.print("Wrote: ");
          Serial.println(qsid[i]);
        }


        Serial.println("writing eeprom pass:");
        for (int i = 0; i < qpass.length(); ++i)
        {
          EEPROM.write(32 + i, qpass[i]);
          Serial.print("Wrote: ");
          Serial.println(qpass[i]);
        }

        Serial.println("writing eeprom nameset:");

        for (int i = 0; i < qnameset.length(); ++i)
        {
          EEPROM.write(96 + i, qnameset[i]);
          Serial.print("Wrote: ");
          Serial.println(qnameset[i]);
        }



        EEPROM.commit();

        content = "{\"Success\":\"saved to eeprom... reset to boot into new wifi\"}";
        statusCode = 200;
        ESP.reset();
      }

      else {
        content = "{\"Error\":\"404 not found\"}";
        statusCode = 404;
        Serial.println("Sending 404");
      }
      server0.sendHeader("Access-Control-Allow-Origin", "*");
      server0.send(statusCode, "application/json", content);

    });


  }
}

void handle_OnConnect()
{
  Serial.println("Client Connected");
  server0.send(200, "text/html", HTML());
}

void handle_ACon()
{
  Serial.println("AC POWER ON");
  AC_status = true;
  ac.next.power = true;
  ac.sendAc();  // Have the IRac class create and send a message.
  server0.send(200, "text/html", HTML());
}

void handle_ACoff()
{
  Serial.println("AC POWER OFF");
  AC_status = false;
  ac.next.power = false;
  ac.sendAc();  // Have the IRac class create and send a message.
  server0.send(200, "text/html", HTML());
}

void handle_heat()
{
  Serial.println("HEAT");
  Mode_status = true;
  ac.next.mode = stdAc::opmode_t::kHeat;  // Run in cool mode initially.
  ac.sendAc();
  

 EEPROM.write(161, 0);
 EEPROM.commit();
 
  server0.send(200, "text/html", HTML());
}

void handle_cool()
{
  Serial.println("COOL");
  Mode_status = false;
  ac.next.mode = stdAc::opmode_t::kCool;  // Run in cool mode initially.
  ac.sendAc();
  EEPROM.write(161, 1);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}



void handle_low()
{
  Serial.println("FAN LOW");
  Fan_status = 1;
  ac.next.fanspeed = stdAc::fanspeed_t::kLow;  // Start the fan at medium.
  ac.sendAc();
  
 EEPROM.write(162, 1);
 EEPROM.commit();
 
  server0.send(200, "text/html", HTML());
}

void handle_medium()
{
  Serial.println("FAN MEDIUM");
  Fan_status = 2;
  ac.next.fanspeed = stdAc::fanspeed_t::kMedium;  // Start the fan at medium.
  ac.sendAc();
   EEPROM.write(162, 2);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_high()
{
  Serial.println("FAN HIGH");
  Fan_status = 0;
  ac.next.fanspeed = stdAc::fanspeed_t::kHigh;  // Start the fan at medium.
  ac.sendAc();
     EEPROM.write(162, 0);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_normal()
{
  Serial.println("NORMAL");
  Turbo_status = false;

  ac.next.turbo = false;  // Don't use any turbo/powerful/etc modes.
  ac.sendAc();
   EEPROM.write(163, 0);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_turbo()
{
  Serial.println("TURBO");
  Turbo_status = true;

  ac.next.turbo = true;  // Don't use any turbo/powerful/etc modes.
  ac.sendAc();
 EEPROM.write(163, 1);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}


void handle_swingOff()
{
  Serial.println("SWING OFF");
  SwingUD_status = 1;

  ac.next.swingv = stdAc::swingv_t::kOff;  // Don't swing the fan up or down.
  ac.sendAc();
  EEPROM.write(164, 1);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_swingLow()
{
  Serial.println("SWING LOW");
  SwingUD_status = 3;
  ac.next.swingv = stdAc::swingv_t::kLow;  // Don't swing the fan up or down.

  ac.sendAc();
     EEPROM.write(164, 3);
 EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

//void handle_swingMed()
//{
//  Serial.println("SWING MED");
//  SwingUD_status = 3;
//  ac.sendAc();
//  server.send(200, "text/html", HTML());
//}

void handle_swingH()
{
  Serial.println("SWING HIGH");
  SwingUD_status = 0;
  ac.next.swingv = stdAc::swingv_t::kHigh;  // Don't swing the fan up or down.
  ac.sendAc();
  EEPROM.write(164, 0);
  EEPROM.commit();
  server0.send(200, "text/html", HTML());
}



void handle_up()
{
  Serial.println("TEMP UP");
  Temp_status = true;
  ac.next.degrees = ac.next.degrees + 1;
  ac.sendAc();
  EEPROM.write(165, ac.next.degrees);
  EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_down()
{
  Serial.println("TEMP DOWN");
  Temp_status = false;
  ac.next.degrees = ac.next.degrees - 1;
  ac.sendAc();
  EEPROM.write(165, ac.next.degrees);
  EEPROM.commit();
  server0.send(200, "text/html", HTML());
}

void handle_NotFound()
{
  server0.send(404, "text/plain", "Not found");
}

String HTML()
{
  String msg = "<!DOCTYPE html> <html>\n";
  msg += "<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  msg += "<title>AC Remote Server</title>\n";
  msg += "<style>html{font-family:Helvetica; display:inline-block; margin:0px auto; text-align:center;}\n";
  msg += "body{margin-top: 15px;} h1{color: #444444; margin: 10px auto 25px;} h2{color: #444444; margin-bottom: 50px; font-size:15px; margin-top: 5px;}  h3{color:#444444; margin-bottom: 0px;} \n";

  msg += ".button{display:block; width:80px; background-color:#f48100; border:none; color:white; padding: 3px 30px; text-decoration:none; font-size:17px; margin: 4px auto 20px; cursor:pointer; border-radius:4px;}\n";

  msg += ".button-on{background-color:#008000;}\n";
  msg += ".button-on:active{background-color:#008000;}\n";
  msg += ".button-off{background-color:#bc0b00;}\n";
  msg += ".button-off:active{background-color:#bc0b00;}\n";

  msg += ".button-heat{background-color:#f48100;}\n";
  msg += ".button-cool{background-color:#20B2AA;}\n";
  msg += ".button-cool:active{background-color:#20B2AA;}\n";

  msg += ".button-normal{background-color:#26282d;}\n";
  msg += ".button-normal:active{background-color:#26282d;}\n";
  msg += ".button-turbo{background-color:#ff0000;}\n";
  msg += ".button-turbo:active{background-color:#ff0000;}\n";



  msg += ".button-speedLow{background-color:#0A6522;}\n";
  msg += ".button-speedLow:active{background-color:#0A6522;}\n";

  msg += ".button-speedMed{background-color:#ffD801;}\n";
  msg += ".button-speedMed:active{background-color:#ffD801;}\n";

  msg += ".button-speedH{background-color:#BF0A30;}\n";
  msg += ".button-speedH:active{background-color:#BF0A30;}\n";



  msg += ".button-swingOff{background-color:#26282d;}\n";
  msg += ".button-swingOff:active{background-color:#26282d;}\n";

  msg += ".button-swingLow{background-color:#0A6522;}\n";
  msg += ".button-swingLow:active{background-color:#0A6522;}\n";

  msg += ".button-swingH{background-color:#BF0A30;}\n";
  msg += ".button-swingH:active{background-color:#BF0A30;}\n";


  msg += ".buttonT{display:block; width:60px; background-color:#f48100; border:none; color:white; padding: 3px 15px; text-decoration:none; font-size:12px; margin: 4px auto 12px; cursor:pointer; border-radius:4px;}\n";
  msg += ".button-up{background-color:#4D4E4F;}\n";
  msg += ".button-up:active{background-color:#4D4E4F;}\n";
  msg += ".button-down{background-color:#4D4E4F;}\n";
  msg += ".button-down:active{background-color:#4D4E4F;}\n";

  msg += "</style>\n";
  msg += "</head>\n";
  msg += "<body>\n";
  msg += "<p><h1>AC Remote Server</h1></a>\n";



  msg += "<a><h3>";
  msg += DeviceName;
  msg += "</h3></a>\n";

  msg += "<a><h2>";
  msg += "<a>IP: </a>";
  msg += LocalIP;
  msg += "</h2></a>\n";

  if (AC_status == false)
  {
    msg += "<a>AC POWER: OFF</n><a class=\"button button-on\" href=\"/ACon\">ON</a>\n";
  }
  else
  {
    msg += "<a>AC POWER: ON</a><a class=\"button button-off\" href=\"/ACoff\">OFF</a>\n";
  }

  if (Mode_status == false)
  {
    msg += "<a>MODE: COOL</a><a class=\"button button-heat\" href=\"/heat\">HEAT</a>\n";
  }
  else
  {
    msg += "<a>MODE: HEAT</a><a class=\"button button-cool\" href=\"/cool\">COOL</a>\n";
  }


  if (Fan_status == 0)
  {
    msg += "<a>FAN SPEED: HIGH</a><a class=\"button button-speedLow\" href=\"/low\">LOW</a>\n";
  }
  if (Fan_status == 1)
  {
    msg += "<a>FAN SPEED: LOW</a><a class=\"button button-speedMed\"href=\"/medium\">MED</a>\n";
  }
  if (Fan_status == 2)
  {
    msg += "<a>FAN SPEED: MEDIUM</a><a class=\"button button-speedH\" href=\"/high\">HIGH</a>\n";
  }


  if (Turbo_status == false)
  {
    msg += "<a>NORMAL</a><a class=\"button button-turbo\" href=\"/turbo\">TURBO</a>\n";
  }
  else
  {
    msg += "<a>TURBO</a><a class=\"button button-normal\" href=\"/normal\">NORMAL</a>\n";
  }


  if (SwingUD_status == 0)
  {
    msg += "<a>SWING: HIGH</a><a class=\"button button-swingOff\" href=\"/swingOff\">OFF</a>\n";
  }
  if (SwingUD_status == 1)
  {
    msg += "<a>SWING: OFF</a><a class=\"button button-swingLow\"href=\"/swingLow\">LOW</a>\n";
  }
  //  if(SwingUD_status==2)
  //  {
  //    msg+="<a>SWING: LOW</a><a class=\"button button-swingMed\" href=\"/swingMed\">MED</a>\n";
  //  }
  if (SwingUD_status == 3)
  {
    msg += "<a>SWING: LOW</a><a class=\"button button-swingH\" href=\"/swingH\">HIGH</a>\n";
  }



  if (Temp_status == false)
  {
    if (ac.next.degrees >= 30) {
      ac.next.degrees = 30;
    }
    if (ac.next.degrees <= 16) {
      ac.next.degrees = 16;
    }

    msg += "<a>TEMP:</a>";
    msg += ac.next.degrees;

    msg += "<a class=\"buttonT button-up\" href=\"/up\">UP</a>\n";
    msg += "<a class=\"buttonT button-down\" href=\"/down\">DOWN</a>\n";

  }
  else
  {
    if (ac.next.degrees >= 30) {
      ac.next.degrees = 30;
    }
    if (ac.next.degrees <= 16) {
      ac.next.degrees = 16;
    }

    msg += "<a>TEMP:</a>";
    msg += ac.next.degrees;

    msg += "<a class=\"buttonT button-up\" href=\"/up\">UP</a>\n";
    msg += "<a class=\"buttonT button-down\" href=\"/down\">DOWN</a>\n";
  }
 
  msg += "<p></a>";
  msg += "<a>PROTOCOL</a>\n";
  msg += "<form action=\"/set\">";

  msg += "<span class=\"slider round\"></span> <select name=\"pl\">  <option value=\"0\"";
  
  if (ProList == 0)
  msg += " selected";
  msg += ">LG</option><option value=\"1\"";

  
  if (ProList == 1) 
  msg += " selected";
  msg += ">COOLIX</option><option value=\"2\"";


  if (ProList == 2) 
  msg += " selected";
  msg += ">DAIKIN</option> <option value=\"3\"";

  if (ProList == 3) 
  msg += " selected";
  msg += ">KELVINATOR</option> <option value=\"4\"";

  if (ProList == 4) 
  msg += " selected";
  msg += ">MITSUBISHI_AC</option> <option value=\"5\"";

  if (ProList == 5) 
  msg += " selected";
  msg += ">GREE</option> <option value=\"6\"";

  if (ProList == 6) 
  msg += " selected";
  msg += ">ARGO</option> <option value=\"7\"";

  if (ProList == 7) 
  msg += " selected";
  msg += ">TROTEC</option> <option value=\"8\"";

  if (ProList == 8) 
  msg += " selected";
  msg += ">TOSHIBA_AC</option> <option value=\"9\"";

  if (ProList == 9) 
  msg += " selected";
  msg += ">FUJITSU_AC</option> <option value=\"10\"";


  if (ProList == 10) 
  msg += " selected";
  msg += ">MIDEA</option> <option value=\"11\"";

  if (ProList == 11) 
  msg += " selected";
  msg += ">HAIER_AC</option> <option value=\"12\"";

  if (ProList == 12) 
  msg += " selected";
  msg += ">HITACHI_AC</option> <option value=\"13\"";

  if (ProList == 13) 
  msg += " selected";
  msg += ">HITACHI_AC1</option> <option value=\"14\"";

  if (ProList == 14) 
  msg += " selected";
  msg += ">HAIER_AC_YRW02</option> <option value=\"15\"";

  if (ProList == 15) 
  msg += " selected";
  msg += ">WHIRLPOOL_AC</option> <option value=\"16\"";


    if (ProList == 16) 
  msg += " selected";
  msg += ">SAMSUNG_AC</option> <option value=\"17\"";

  if (ProList == 17) 
  msg += " selected";
  msg += ">ELECTRA_AC</option> <option value=\"18\"";


    if (ProList == 18) 
  msg += " selected";
  msg += ">PANASONIC_AC</option> <option value=\"19\"";


    if (ProList == 19) 
  msg += " selected";
  msg += ">LG2</option> <option value=\"20\"";


    if (ProList == 20) 
  msg += " selected";
  msg += ">DAIKIN2</option> <option value=\"21\"";


    if (ProList == 21) 
  msg += " selected";
  msg += ">VESTEL_AC</option> <option value=\"22\"";


    if (ProList == 22) 
  msg += " selected";
  msg += ">TECO</option> <option value=\"23\"";


    if (ProList == 23) 
  msg += " selected";
  msg += ">TCL112AC</option> <option value=\"24\"";


    if (ProList == 24) 
  msg += " selected";
  msg += ">MITSUBISHI_HEAVY_88</option> <option value=\"25\"";


    if (ProList == 25) 
  msg += " selected";
  msg += ">MITSUBISHI_HEAVY_152</option> <option value=\"26\"";


    if (ProList == 26) 
  msg += " selected";
  msg += ">DAIKIN216</option> <option value=\"27\"";


      if (ProList == 27) 
  msg += " selected";
  msg += ">SHARP_AC</option> <option value=\"28\"";
 
    if (ProList == 28) 
  msg += " selected";
  msg += ">GOODWEATHER</option> <option value=\"29\"";

     if (ProList == 29) 
  msg += " selected";
  msg += ">DAIKIN160</option> <option value=\"30\"";
 
     if (ProList == 30) 
  msg += " selected";
  msg += ">NEOCLIMA</option> <option value=\"31\"";

     if (ProList == 31) 
  msg += " selected";
  msg += ">DAIKIN176</option> <option value=\"32\"";

    if (ProList == 32) 
  msg += " selected";
  msg += ">DAIKIN128</option> <option value=\"33\"";

    if (ProList == 33) 
  msg += " selected";
  msg += ">AMCOR</option> <option value=\"34\"";

    if (ProList == 34) 
  msg += " selected";
  msg += ">DAIKIN152</option> <option value=\"35\"";

    if (ProList == 35) 
  msg += " selected";
  msg += ">MITSUBISHI136</option> <option value=\"36\"";

    if (ProList == 36) 
  msg += " selected";
  msg += ">MITSUBISHI112</option> <option value=\"37\"";

    if (ProList == 37) 
  msg += " selected";
  msg += ">HITACHI_AC424</option> <option value=\"38\"";

    if (ProList == 38) 
  msg += " selected";
  msg += ">DAIKIN64</option> <option value=\"39\"";

      if (ProList == 39) 
  msg += " selected";
  msg += ">AIRWELL</option> <option value=\"40\"";

    if (ProList == 40) 
  msg += " selected";
  msg += ">DELONGHI_AC</option> <option value=\"41\"";

    if (ProList == 41) 
  msg += " selected";
  msg += ">CARRIER_AC64</option> <option value=\"42\"";

    if (ProList == 42) 
  msg += " selected";
  msg += ">HITACHI_AC344</option> <option value=\"43\"";

    if (ProList == 43) 
  msg += " selected";
  msg += ">CORONA_AC</option> <option value=\"44\"";

    if (ProList == 44) 
  msg += " selected";
  msg += ">SANYO_AC</option> <option value=\"45\"";

    if (ProList == 45) 
  msg += " selected";
  msg += ">VOLTAS</option> <option value=\"46\"";

    if (ProList == 46) 
  msg += " selected";
  msg += ">TRANSCOLD</option> <option value=\"47\"";

    if (ProList == 47) 
  msg += " selected";
  msg += ">TECHNIBEL_AC</option> <option value=\"48\"";

    if (ProList == 48) 
  msg += " selected";
  msg += ">MIRAGE</option> <option value=\"49\"";

    if (ProList == 49) 
  msg += " selected";
  msg += ">PANASONIC_AC32</option> <option value=\"50\"";

    if (ProList == 50) 
  msg += " selected";
  msg += ">ECOCLIM</option> <option value=\"51\"";

    if (ProList == 51) 
  msg += " selected";
  msg += ">TRUMA</option> <option value=\"52\"";

    if (ProList == 52) 
  msg += " selected";
  msg += ">HAIER_AC176</option> <option value=\"53\"";
    if (ProList == 53) 
  msg += " selected";
  msg += ">TEKNOPOINT</option> <option value=\"54\"";

    if (ProList == 54) 
  msg += " selected";
  msg += ">KELON</option> <option value=\"55\"";

    if (ProList == 55) 
  msg += " selected";
  msg += ">TROTEC_3550</option> <option value=\"56\"";

    if (ProList == 56) 
  msg += " selected";
  msg += ">SANYO_AC88</option> <option value=\"57\"";

    if (ProList == 57) 
  msg += " selected";
  msg += ">RHOSS</option> <option value=\"58\"";
 
  if (ProList == 58) 
  msg += " selected";
  msg += ">AIRTON</option> </select>";

  msg += "<a> </p>";
  msg += "<div class=\"btn\"><input type=\"submit\" value=\"SET\"> </div></form>";


  
//  if (Test_status == false)
//  {
//  msg += "<a class=\"buttonT button-down\" href=\"/test\">TEST</a>\n";
//  }
//   if (Test_status == true)
//  {
//  msg += "<a class=\"buttonT button-down\" href=\"/stoptest\">STOP</a>\n";
//  }

  
  msg += "</body>\n";
  msg += "</html>\n";


  return msg;
}

void setProto(){
    if (server0.hasArg("pl")) {     
      ProList = server0.arg("pl").toInt();
    }
    
      server0.send(200, "text/html", HTML());
       
if (ProList == 0) {
   ac.next.protocol = decode_type_t::LG;  // Set a protocol to use.
  ac.sendAc();
}

if (ProList == 1) {
   ac.next.protocol = decode_type_t::COOLIX;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 2) {
     ac.next.protocol = decode_type_t::DAIKIN;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 3) {
    ac.next.protocol = decode_type_t::KELVINATOR;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 4) {
    ac.next.protocol = decode_type_t::MITSUBISHI_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 5) {
     ac.next.protocol = decode_type_t::GREE;  // Set a protocol to use.
  ac.sendAc();
}

if (ProList == 6) {
     ac.next.protocol = decode_type_t::ARGO;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 7) {
     ac.next.protocol = decode_type_t::TROTEC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 8) {
     ac.next.protocol = decode_type_t::TOSHIBA_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 9) {
     ac.next.protocol = decode_type_t::FUJITSU_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 10) {
     ac.next.protocol = decode_type_t::MIDEA;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 11) {
     ac.next.protocol = decode_type_t::HAIER_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 12) {
     ac.next.protocol = decode_type_t::HITACHI_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 13) {
     ac.next.protocol = decode_type_t::HITACHI_AC1;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 14) {
     ac.next.protocol = decode_type_t::HAIER_AC_YRW02;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 15) {
     ac.next.protocol = decode_type_t::WHIRLPOOL_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 16) {
     ac.next.protocol = decode_type_t::SAMSUNG_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 17) {
     ac.next.protocol = decode_type_t::ELECTRA_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 18) {
     ac.next.protocol = decode_type_t::PANASONIC_AC;  // Set a protocol to use.
  ac.sendAc();
}if (ProList == 19) {
     ac.next.protocol = decode_type_t::LG2;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 20) {
     ac.next.protocol = decode_type_t::DAIKIN2;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 21) {
     ac.next.protocol = decode_type_t::VESTEL_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 22) {
     ac.next.protocol = decode_type_t::TECO;  // Set a protocol to use.
  ac.sendAc();
}if (ProList == 23) {
     ac.next.protocol = decode_type_t::TCL112AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 24) {
     ac.next.protocol = decode_type_t::MITSUBISHI_HEAVY_88;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 25) {
     ac.next.protocol = decode_type_t::MITSUBISHI_HEAVY_152;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 26) {
     ac.next.protocol = decode_type_t::DAIKIN216;  // Set a protocol to use.
  ac.sendAc();
}if (ProList == 27) {
     ac.next.protocol = decode_type_t::SHARP_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 28) {
     ac.next.protocol = decode_type_t::GOODWEATHER;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 29) {
     ac.next.protocol = decode_type_t::DAIKIN160;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 30) {
     ac.next.protocol = decode_type_t::NEOCLIMA;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 31) {
     ac.next.protocol = decode_type_t::DAIKIN176;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 32) {
     ac.next.protocol = decode_type_t::DAIKIN128;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 33) {
     ac.next.protocol = decode_type_t::AMCOR;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 34) {
     ac.next.protocol = decode_type_t::DAIKIN152;  // Set a protocol to use.
  ac.sendAc();
}

if (ProList == 35) {
     ac.next.protocol = decode_type_t::MITSUBISHI136;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 36) {
     ac.next.protocol = decode_type_t::MITSUBISHI112;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 37) {
     ac.next.protocol = decode_type_t::HITACHI_AC424;  // Set a protocol to use.
  ac.sendAc();
}

if (ProList == 38) {
     ac.next.protocol = decode_type_t::DAIKIN64;  // Set a protocol to use.
  ac.sendAc();
}

if (ProList == 39) {
     ac.next.protocol = decode_type_t::AIRWELL;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 40) {
     ac.next.protocol = decode_type_t::DELONGHI_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 41) {
     ac.next.protocol = decode_type_t::CARRIER_AC64;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 42) {
     ac.next.protocol = decode_type_t::HITACHI_AC344;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 43) {
     ac.next.protocol = decode_type_t::CORONA_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 44) {
     ac.next.protocol = decode_type_t::SANYO_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 45) {
     ac.next.protocol = decode_type_t::VOLTAS;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 46) {
     ac.next.protocol = decode_type_t::TRANSCOLD;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 47) {
     ac.next.protocol = decode_type_t::TECHNIBEL_AC;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 48) {
     ac.next.protocol = decode_type_t::MIRAGE;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 49) {
     ac.next.protocol = decode_type_t::PANASONIC_AC32;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 50) {
     ac.next.protocol = decode_type_t::ECOCLIM;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 51) {
     ac.next.protocol = decode_type_t::TRUMA;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 52) {
     ac.next.protocol = decode_type_t::HAIER_AC176;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 53) {
     ac.next.protocol = decode_type_t::TEKNOPOINT;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 54) {
     ac.next.protocol = decode_type_t::KELON;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 55) {
     ac.next.protocol = decode_type_t::TROTEC_3550;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 56) {
     ac.next.protocol = decode_type_t::SANYO_AC88;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 57) {
     ac.next.protocol = decode_type_t::RHOSS;  // Set a protocol to use.
  ac.sendAc();
}
if (ProList == 58) {
     ac.next.protocol = decode_type_t::AIRTON;  // Set a protocol to use.
  ac.sendAc();
}

 EEPROM.write(160, ProList);
 EEPROM.commit();
}
